package fxKerho;

import static fxKerho.TietueDialogController.getFieldId; 
import java.awt.Desktop;
import java.io.IOException;
import java.io.PrintStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Collection;
import java.util.List;
import java.util.ResourceBundle;

import fi.jyu.mit.fxgui.ComboBoxChooser;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.StringGrid;
import fi.jyu.mit.fxgui.TextAreaOutputStream;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TableView;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.GridPane;
import kerho.Harrastus;
import kerho.Jasen;
import kerho.Kerho;
import kerho.SailoException;

/**
 * Luokka kerhon käyttöliittymän tapahtumien hoitamiseksi.
 * 
 * @author vesal
 * @version 31.12.2015
 * @version 5.1.2015
 * @version 5.2.2017 - lisätty jäsenien käsittely 
 * @version 5.2.2017 - lisätty harrastusten käsittely 
 */
public class KerhoGUIController implements Initializable {

    @FXML private TextField hakuehto;
    @FXML private ComboBoxChooser<String> cbKentat;
    @FXML private Label labelVirhe;
    @FXML private ScrollPane panelJasen;
    @FXML private GridPane gridJasen;
    @FXML private ListChooser<Jasen> chooserJasenet;
    @FXML private StringGrid<Harrastus> tableHarrastukset;
    
    
    @Override
    public void initialize(URL url, ResourceBundle bundle) {
        alusta();      
    }

    
    @FXML private void handleHakuehto() {
        hae(0); 
    }
    
    
    @FXML private void handleTallenna() {
        tallenna();
    }
    
    
    @FXML private void handleAvaa() {
        avaa();
    }
    
    
    @FXML private void handleTulosta() {
        TulostusController tulostusCtrl = TulostusController.tulosta(null); 
        tulostaValitut(tulostusCtrl.getTextArea()); 
    } 

    
    @FXML private void handleLopeta() {
        tallenna();
        Platform.exit();
    } 

    
    @FXML private void handleUusiJasen() {
        uusiJasen();
    }
    
    
    @FXML private void handleMuokkaaJasen() {
        muokkaa(kentta);
    }
    
    
    @FXML private void handlePoistaJasen() {
        poistaJasen();
    }
    
     
    @FXML private void handleUusiHarrastus() {
        uusiHarrastus(); 
    }
    

    @FXML private void handleMuokkaaHarrastus() {
        ModalController.showModal(KerhoGUIController.class.getResource("HarrastusDialogView.fxml"), "Harrastus", null, "");
    }
    

    @FXML private void handlePoistaHarrastus() {
        poistaHarrastus();
    }
    

    @FXML private void handleApua() {
        avustus();
    }
    

    @FXML private void handleTietoja() {
        // Dialogs.showMessageDialog("Ei osata vielä tietoja");
        ModalController.showModal(KerhoGUIController.class.getResource("AboutView.fxml"), "Kerho", null, "");
    }
    

//===========================================================================================    
// Tästä eteenpäin ei käyttöliittymään suoraan liittyvää koodia    
    
    private String kerhonnimi = "kelmit";
    private Kerho kerho;
    private Jasen jasenKohdalla;
    private TextField edits[]; 
    private int kentta = 0; 
    private static Jasen apujasen = new Jasen(); 
    private static Harrastus apuharrastus = new Harrastus(); 
    
    
    /**
     * Tekee tarvittavat muut alustukset, nyt vaihdetaan GridPanen tilalle
     * yksi iso tekstikenttä, johon voidaan tulostaa jäsenten tiedot.
     * Alustetaan myös jäsenlistan kuuntelija 
     */
    protected void alusta() {
        chooserJasenet.clear();
        chooserJasenet.addSelectionListener(e -> naytaJasen());
        
        cbKentat.clear(); 
        for (int k = apujasen.ekaKentta(); k < apujasen.getKenttia(); k++) 
            cbKentat.add(apujasen.getKysymys(k), null); 
        cbKentat.getSelectionModel().select(0); 
        
        edits = TietueDialogController.luoKentat(gridJasen, apujasen);  
        for (TextField edit: edits)  
            if ( edit != null ) {  
                edit.setEditable(false);  
                edit.setOnMouseClicked(e -> { if ( e.getClickCount() > 1 ) muokkaa(getFieldId(e.getSource(),0)); });  
                edit.focusedProperty().addListener((a,o,n) -> kentta = getFieldId(edit,kentta));
                edit.setOnKeyPressed( e -> {if ( e.getCode() == KeyCode.F2 ) muokkaa(kentta);}); 
            }    
        // alustetaan harrastustaulukon otsikot 
        int eka = apuharrastus.ekaKentta(); 
        int lkm = apuharrastus.getKenttia(); 
        String[] headings = new String[lkm-eka]; 
        for (int i=0, k=eka; k<lkm; i++, k++) headings[i] = apuharrastus.getKysymys(k); 
        tableHarrastukset.initTable(headings); 
        tableHarrastukset.setColumnResizePolicy(TableView.CONSTRAINED_RESIZE_POLICY); 
        tableHarrastukset.setEditable(false); 
        tableHarrastukset.setPlaceholder(new Label("Ei vielä harrastuksia")); 
         
        // Tämä on vielä huono, ei automaattisesti muutu jos kenttiä muutetaan. 
        tableHarrastukset.setColumnSortOrderNumber(1); 
        tableHarrastukset.setColumnSortOrderNumber(2); 
        tableHarrastukset.setColumnWidth(1, 60); 
        tableHarrastukset.setColumnWidth(2, 60); 
        
        tableHarrastukset.setOnMouseClicked( e -> { if ( e.getClickCount() > 1 ) muokkaaHarrastusta(); } );
        tableHarrastukset.setOnKeyPressed( e -> {if ( e.getCode() == KeyCode.F2 ) muokkaaHarrastusta();}); 
    }

    
    private void naytaVirhe(String virhe) {
        if ( virhe == null || virhe.isEmpty() ) {
            labelVirhe.setText("");
            labelVirhe.getStyleClass().removeAll("virhe");
            return;
        }
        labelVirhe.setText(virhe);
        labelVirhe.getStyleClass().add("virhe");
    }
    
    
    private void setTitle(String title) {
        ModalController.getStage(hakuehto).setTitle(title);
    }
    
    
    /**
     * Alustaa kerhon lukemalla sen valitun nimisestä tiedostosta
     * @param nimi tiedosto josta kerhon tiedot luetaan
     * @return null jos onnistuu, muuten virhe tekstinä
     */
    protected String lueTiedosto(String nimi) {
        kerhonnimi = nimi;
        setTitle("Kerho - " + kerhonnimi);
        try {
            kerho.lueTiedostosta(nimi);
            hae(0);
            return null;
        } catch (SailoException e) {
            hae(0);
            String virhe = e.getMessage(); 
            if ( virhe != null ) Dialogs.showMessageDialog(virhe);
            return virhe;
        }
     }

    
    /**
     * Kysytään tiedoston nimi ja luetaan se
     * @return true jos onnistui, false jos ei
     */
    public boolean avaa() {
        String uusinimi = KerhonNimiController.kysyNimi(null, kerhonnimi);
        if (uusinimi == null) return false;
        lueTiedosto(uusinimi);
        return true;
    }

    
    /**
     * Tietojen tallennus
     * @return null jos onnistuu, muuten virhe tekstinä
     */
    private String tallenna() {
        try {
            kerho.tallenna();
            return null;
        } catch (SailoException ex) {
            Dialogs.showMessageDialog("Tallennuksessa ongelmia! " + ex.getMessage());
            return ex.getMessage();
        }
    }


    /**
     * Tarkistetaan onko tallennus tehty
     * @return true jos saa sulkea sovelluksen, false jos ei
     */
    public boolean voikoSulkea() {
        tallenna();
        return true;
    }
    
    
    /**
     * Näyttää listasta valitun jäsenen tiedot tekstikenttiin. 
     */
    protected void naytaJasen() {
        jasenKohdalla = chooserJasenet.getSelectedObject();
        if (jasenKohdalla == null) return;
        
        TietueDialogController.naytaTietue(edits, jasenKohdalla); 
        naytaHarrastukset(jasenKohdalla);
    }


    /**
     * Hakee jäsenten tiedot listaan
     * @param jnr jäsenen numero, joka aktivoidaan haun jälkeen  
     */
    protected void hae(int jnr) {
        int jnro = jnr; // jnro jäsenen numero, joka aktivoidaan haun jälkeen 
        if ( jnro <= 0 ) { 
            Jasen kohdalla = jasenKohdalla; 
            if ( kohdalla != null ) jnro = kohdalla.getTunnusNro(); 
        }
        
        int k = cbKentat.getSelectionModel().getSelectedIndex() + apujasen.ekaKentta(); 
        String ehto = hakuehto.getText(); 
        if (ehto.indexOf('*') < 0) ehto = "*" + ehto + "*"; 
        
        chooserJasenet.clear();

        int index = 0;
        Collection<Jasen> jasenet;
        try {
            jasenet = kerho.etsi(ehto, k);
            int i = 0;
            for (Jasen jasen:jasenet) {
                if (jasen.getTunnusNro() == jnro) index = i;
                chooserJasenet.add(jasen.getNimi(), jasen);
                i++;
            }
        } catch (SailoException ex) {
            Dialogs.showMessageDialog("Jäsenen hakemisessa ongelmia! " + ex.getMessage());
        }
        chooserJasenet.setSelectedIndex(index); // tästä tulee muutosviesti joka näyttää jäsenen
    }


    /**
     * Luo uuden jäsenen jota aletaan editoimaan 
     */
    protected void uusiJasen() {
        try {
            Jasen uusi = new Jasen();
            uusi = TietueDialogController.kysyTietue(null, uusi, 1);      
            if ( uusi == null ) return;
            uusi.rekisteroi();
            kerho.lisaa(uusi);
            hae(uusi.getTunnusNro());
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ongelmia uuden luomisessa " + e.getMessage());
            return;
        }
    }
    
    
    private void naytaHarrastukset(Jasen jasen) {
        tableHarrastukset.clear();
        if ( jasen == null ) return;
        
        try {
            List<Harrastus> harrastukset = kerho.annaHarrastukset(jasen);
            if ( harrastukset.size() == 0 ) return;
            for (Harrastus har: harrastukset)
                naytaHarrastus(har);
        } catch (SailoException e) {
            naytaVirhe(e.getMessage());
        } 
    }

    
    private void naytaHarrastus(Harrastus har) {
        int kenttia = har.getKenttia(); 
        String[] rivi = new String[kenttia-har.ekaKentta()]; 
        for (int i=0, k=har.ekaKentta(); k < kenttia; i++, k++) 
            rivi[i] = har.anna(k); 
        tableHarrastukset.add(har,rivi);
    }
    
    
    /**
     * Tekee uuden tyhjän harrastuksen editointia varten
     */
    private void uusiHarrastus() {
        if ( jasenKohdalla == null ) return;
        try {
            Harrastus uusi = new Harrastus(jasenKohdalla.getTunnusNro());
            uusi = TietueDialogController.kysyTietue(null, uusi, 0);
            if ( uusi == null ) return;
            uusi.rekisteroi();
            kerho.lisaa(uusi);
            naytaHarrastukset(jasenKohdalla); 
            tableHarrastukset.selectRow(1000);  // järjestetään viimeinen rivi valituksi
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Lisääminen epäonnistui: " + e.getMessage());
        }
    }
    
    
    private void muokkaaHarrastusta() {
        int r = tableHarrastukset.getRowNr();
        if ( r < 0 ) return; // klikattu ehkä otsikkoriviä
        Harrastus har = tableHarrastukset.getObject();
        if ( har == null ) return;
        int k = tableHarrastukset.getColumnNr()+har.ekaKentta();
        try {
            har = TietueDialogController.kysyTietue(null, har.clone(), k);
            if ( har == null ) return;
            kerho.korvaaTaiLisaa(har); 
            naytaHarrastukset(jasenKohdalla); 
            tableHarrastukset.selectRow(r);  // järjestetään sama rivi takaisin valituksi
        } catch (CloneNotSupportedException  e) { /* clone on tehty */  
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ongelmia lisäämisessä: " + e.getMessage());
        }
    }

    
    private void muokkaa(int k) { 
        if ( jasenKohdalla == null ) return; 
        try { 
            Jasen jasen; 
            jasen = TietueDialogController.kysyTietue(null, jasenKohdalla.clone(), k);     
            if ( jasen == null ) return; 
            kerho.korvaaTaiLisaa(jasen); 
            hae(jasen.getTunnusNro()); 
        } catch (CloneNotSupportedException e) { 
            // 
        } catch (SailoException e) { 
            Dialogs.showMessageDialog(e.getMessage()); 
        } 
    }     
    
    
    /**
     * @param kerho Kerho jota käytetään tässä käyttöliittymässä
     */
    public void setKerho(Kerho kerho) {
        this.kerho = kerho;
        naytaJasen();
    }

    


    /**
     * Poistetaan harrastustaulukosta valitulla kohdalla oleva harrastus. 
     */
    private void poistaHarrastus() {
        int rivi = tableHarrastukset.getRowNr();
        if ( rivi < 0 ) return;
        Harrastus harrastus = tableHarrastukset.getObject();
        if ( harrastus == null ) return;
        kerho.poistaHarrastus(harrastus);
        naytaHarrastukset(jasenKohdalla);
        int harrastuksia = tableHarrastukset.getItems().size(); 
        if ( rivi >= harrastuksia ) rivi = harrastuksia -1;
        tableHarrastukset.getFocusModel().focus(rivi);
        tableHarrastukset.getSelectionModel().select(rivi);
    }


    /*
     * Poistetaan listalta valittu jäsen
     */
    private void poistaJasen() {
        Jasen jasen = jasenKohdalla;
        if ( jasen == null ) return;
        if ( !Dialogs.showQuestionDialog("Poisto", "Poistetaanko jäsen: " + jasen.getNimi(), "Kyllä", "Ei") )
            return;
        kerho.poista(jasen);
        int index = chooserJasenet.getSelectedIndex();
        hae(0);
        chooserJasenet.setSelectedIndex(index);
    }

    
    /**
     * Näytetään ohjelman suunnitelma erillisessä selaimessa.
     */
    private void avustus() {
        Desktop desktop = Desktop.getDesktop();
        try {
            URI uri = new URI("https://tim.jyu.fi/view/kurssit/tie/ohj2/2019k/ht/vesal");
            desktop.browse(uri);
        } catch (URISyntaxException e) {
            return;
        } catch (IOException e) {
            return;
        }
    }

    
    /**
     * Tulostaa jäsenen tiedot
     * @param os tietovirta johon tulostetaan
     * @param jasen tulostettava jäsen
     */
    public void tulosta(PrintStream os, final Jasen jasen) {
        os.println("----------------------------------------------");
        jasen.tulosta(os);
        os.println("----------------------------------------------");
        try {
            List<Harrastus> harrastukset = kerho.annaHarrastukset(jasen);
            for (Harrastus har:harrastukset) 
                har.tulosta(os);     
        } catch (SailoException ex) {
            Dialogs.showMessageDialog("Harrastusten hakemisessa ongelmia! " + ex.getMessage());
        }      
    }
    
    
    /**
     * Tulostaa listassa olevat jäsenet tekstialueeseen
     * @param text alue johon tulostetaan
     */
    public void tulostaValitut(TextArea text) {
        try (PrintStream os = TextAreaOutputStream.getTextPrintStream(text)) {
            os.println("Tulostetaan kaikki jäsenet");
            for (Jasen jasen: chooserJasenet.getObjects()) { 
                tulosta(os, jasen);
                os.println("\n\n");
            }
        }
    }
}
